local NekoLoot = LibStub("AceAddon-3.0"):GetAddon("NekoLoot")

local AceConfDia = LibStub("AceConfigDialog-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("NekoLoot")

local MODNAME = "NekoLootRoll"
local NekoLootRoll = NekoLoot:NewModule(MODNAME)

local options, db, rollAnc, rollFrames = nil, nil, nil, {}

------
-- flag: client locale is ruRU 
local ruRU = (GetLocale() == "ruRU")

------
-- defaults
local defaults = {
	profile = {
		iconSize = 24,
		height = 22,
		width = 240,
		scale = 1.0,
		alpha = 1.0,
		strata = "DIALOG",
			
		parent = UIParent,
		anchorPoint = "LEFT",
		parentAnchor = "LEFT",
		lock = true,
		xOffSet = 200,
		yOffSet = 200,

		tokenBGColor = {0, 0, 0, .9},
		tokenBorderColor = {0.7, 0.7, 0.7, 1},
		
		bopColor = {1, 0.3, 0.1, 1},
		boeColor = {0.3, 1, 0.3, 1},
		
		bindFont = "Fonts\\FRIZQT__.TTF",
		bindSize = 13,
		bindFlag = "THICKOUTLINE",
		bindColor = { 1, 0.8, 0 },
		
		lootSize = 16,
	},
}


------
-- table roll types - 0 = pass, 1 = need, 2 = greed, 3 = diss, 4 = auto pass
local rollTypes = {"need", "greed", "dis", "passA", [0] = "pass"}

------
-- Get and set magic
local getOpt, setOpt, getColor, setColor, getEnabled, setEnabled
do
	function getOpt(info)
		-- db.profile.GROUP.KEY --> GROUP is penultimate and KEY is ultimate arg of info
		return db[info[#info]]
	end
	
	function setOpt(info, value)
		--db.profile.GROUP.KEY --> GROUP is penultimate and KEY is ultimate arg of info
		db[info[#info]] = value
		NekoLootRoll:ApplySettings()
	end

	function getColor(info)
		return unpack(getOpt(info))
	end
	
	function setColor(info, r, g, b, a)
		setOpt(info, {r, g, b, a})
	end

	function getEnabled(info)
		return NekoLoot:GetModuleEnabled(MODNAME)
	end

	function setEnabled(info, value)
		return NekoLoot:SetModuleEnabled(MODNAME, value)
	end
end


------
-- Get options table. If not already builded, create it.
local function getOptions()
	if not options then
		options = {
			type = "group", inline = false,
			name = L["Config/LootRoll"],
			desc = L["Config/LootRollDesc"],
			args = {
				toggle = {
					type = "toggle", order = 1, width = "full",
					name = L["Toggle"],
					get = getEnabled,
					set = setEnabled,
				},
				iconSize = {
					type = "range", order = 8,
					name = L["IconSize"],
					min = 10, max = 60, step = 1,
					get = getOpt,
					set = setOpt,
				},
				height = {
					type = "range", order = 9,
					name = L["Height"],
					min = 10, max = 80, step = 1,
					get = getOpt,
					set = setOpt,
				},
				width = {
					type = "range", order = 10,
					name = L["Width"],
					min = 50, max = 400, step = 1,
					get = getOpt,
					set = setOpt,
				},
				scale = {
					type = "range", order = 11,
					name = L["Scale"],
					desc = L["ScaleDesc"],
					min = 0.2, max = 2, step = 0.1,
					get = getOpt,
					set = setOpt,
				},
				alpha = {
					type = "range", order = 12,
					name = L["Alpha"],
					desc = L["AlphaDesc"],
					min = 0.1, max = 1, step = 0.1,
					get = getOpt,
					set = setOpt,
				},
				strata = {
					type = "select", order = 13, style = "dropdown",
					name = L["Strata"],
					desc = L["StrataDesc"],
					values = NekoLoot.strataKeys,
					get = getOpt,
					set = setOpt,
				},

				alignmentHeader = {
					type = "header", order = 20,
					name = L["PositionAlignment"],
				},
				parentAnchor = {
					type = "select", order = 21, style = "dropdown",
					name = L["ParentAnchor"],
					desc = L["ParentAnchorDesc"],
					values = NekoLoot.anchorKeys,
					get = getOpt,
					set = setOpt,
				},
				anchorPoint = {
					type = "select", order = 22, style = "dropdown",
					name = L["AnchorPoint"],
					desc = L["AnchorPointDesc"],
					values = NekoLoot.anchorKeys,
					get = getOpt,
					set = setOpt,
				},

				lockSpace = {
					type = "description", order = 25,
					name = "",
				},
				unlock = {
					type = "execute", order = 26,
					name = L["ToggleFrameLock"],
					desc = L["ToggleFrameLockDesc"],
					func = function()
						NekoLoot:ToggleLock(true)
					end,
				},
				lock = {
					type = "toggle", order = 27,
					name = L["Lock"],
					desc = L["LockDesc"],
					get = getOpt,
					set = setOpt,
				},

				offsetSpace = {
					type = "description", order = 30,
					name = "",
				},
				xOffSet = {
					type = "range", order = 31, width = "double",
					name = L["XOff"],
					desc = L["XOffDesc"],
					min = -800, max = 800, step = 1,
					get = getOpt,
					set = setOpt,
				},
				yOffSet = {
					type = "range", order = 32, width = "double",
					name = L["YOff"],
					desc = L["YOffDesc"],
					min = -800, max = 800, step = 1,
					get = getOpt,
					set = setOpt,
				},
			},
		}
	end
	return options
end


------
-- Adds a roll token to the list of created tokens
local function PushToken(token)
	token:SetPoint("TOPLEFT", next(rollFrames) and rollFrames[#rollFrames] or rollAnc, "BOTTOMLEFT", 0, -4)
	table.insert(rollFrames, token)
end

	
------
function NekoLootRoll:OnInitialize()
	self.db = NekoLoot.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	NekoLoot:SetEnabledState(NekoLoot:GetModuleEnabled(MODNAME))
	NekoLoot:RegisterModuleOptions(MODNAME, getOptions, L["LootRoll"])
	
	rollAnc = NekoLoot:CreatRollAnchor()
	
	local token = NekoLoot:CreateRollToken()
	PushToken(token)
	
	NekoLootRoll:ApplySettings()
end


------
function NekoLootRoll:OnEnable()
	-- disable default rollframe
	UIParent:UnregisterEvent("START_LOOT_ROLL")
	UIParent:UnregisterEvent("CANCEL_LOOT_ROLL")

	NekoLootRoll:RegisterEvent("START_LOOT_ROLL", "LootRollStarted")
	-- Fired when a group loot item is being rolled on.
	NekoLootRoll:RegisterEvent("CANCEL_LOOT_ROLL", "LootRollCanceled")
	-- Fired when a player cancels a roll on an item 
	NekoLootRoll:RegisterEvent("CHAT_MSG_LOOT", "ChatMsgLoot")
end


------
function NekoLootRoll:OnDisable()
	-- disable loot roll frame
	NekoLootRoll:UnregisterAllEvents()

	-- enable default roll frame
	UIParent:RegisterEvent("START_LOOT_ROLL")
	UIParent:RegisterEvent("CANCEL_LOOT_ROLL")
end


------
function NekoLootRoll:ApplySettings()
	db = self.db.profile
	
	rollAnc:SetHeight(db.height)
	rollAnc:SetWidth(db.width)
	rollAnc:SetScale(db.scale)
	rollAnc:SetAlpha(db.alpha)
	rollAnc:SetFrameStrata(db.strata)

	rollAnc:SetMovable(not db.lock)
	rollAnc:EnableMouse(not db.lock)
	
	NekoLoot:ApplySkin(rollAnc, db.tokenBGColor, db.tokenBorderColor)

	-- Allignment
   	rollAnc:ClearAllPoints()
	rollAnc:SetPoint(db.anchorPoint, db.parent, db.parentAnchor, db.xOffSet, db.yOffSet)
	
	for index, rollToken in pairs(rollFrames) do
		NekoLootRoll:ApplySetToRollToken(rollToken)
	end
end


------
function NekoLootRoll:ApplySetToRollToken(rollToken)
	-- hopefully a fix for a bug reportet on corse.com
	if db == nil then return end
	
	rollToken:SetWidth(db.width)
	rollToken:SetHeight(db.height)

	NekoLoot:ApplySkin(rollToken, db.tokenBGColor, db.tokenBorderColor)
	
	-- icon
	icon = rollToken.icon
	icon:SetWidth(db.iconSize)
	icon:SetHeight(db.iconSize)
	
	iconBorder = rollToken.iconBorder
	iconBorder:SetWidth(db.iconSize + 8)
	iconBorder:SetHeight(db.iconSize + 8)
	
	-- font
	rollToken.fsbind:SetFont(db.bindFont, db.bindSize, db.bindFlag)
	rollToken.fsloot:SetHeight(db.lootSize)
end


------
-- 
function NekoLootRoll:ToggleLock(showUI)
	if not db.lock then
		NekoLootRoll:Lock()
	else
		NekoLootRoll:Unlock(showUI)
	end
end

------
-- Make roll anchor moveable
function NekoLootRoll:Unlock(showUI)
	db.lock = false
	rollAnc:Show()
	rollAnc:EnableMouse(true)
	rollAnc:SetMovable(true)
	if rollFrames[1] and not rollFrames[1]:IsVisible() then
		rollFrames[1]:Show()
	end
	
	if showUI then NekoLoot:CreateConfigModeDialog() end
end


------
-- Lock roll anchor
function NekoLootRoll:Lock()
	db.lock = true
	rollAnc:Hide()
	rollAnc:EnableMouse(false)
	rollAnc:SetMovable(false)
	if rollFrames[1] and not rollFrames[1].rollid then
		rollFrames[1]:Hide()
	end
end


------
-- Returns an unsed RollToken. If no unused token is found, one is created.
function GetToken()
	for _, token in ipairs(rollFrames) do
		if not token.rollid then 
			return token
		end
	end

	local token = NekoLoot:CreateRollToken()
	NekoLootRoll:ApplySetToRollToken(token)
	PushToken(token)
	return token
end


------
-- Set all Colors of a roll token depend on item quality and binding.
--
-- token			: token itself
-- quality			: imte quality
local function SetTokenColors(token, quality)
	local r, g, b = GetItemQualityColor(quality)
	token.fsloot:SetVertexColor(r, g, b)
	token:SetBackdropBorderColor(r, g, b, 1)
	token.iconBorder:SetBackdropBorderColor(r, g, b, 1)
	token.status:SetStatusBarColor(r, g, b, .7)
end


------
-- Handle START_LOOT_ROLL event
-- Try to get unused roll frame otherwise creates a new one.
-- Sets rollid of the item and time elapsed.
-- Sets some additional infos and the colors.
--
-- event			: event that should be handled
-- rollid			: rollid of the item
-- time				: time elapsed until roll is cancelled
function NekoLootRoll:LootRollStarted(event, rollid, time)
	local token = GetToken()
	token.rollid = rollid
	token.time = time
	for index in pairs(token.rolls) do 
		token.rolls[index] = nil
	end
	
	token.need:SetText(0)
	token.greed:SetText(0)
	token.pass:SetText(0)
	token.dis:SetText(0)

	local icon, name, count, quality, bop, canNeed, canGreed, canDis = GetLootRollItemInfo(rollid)
	token.icon:SetNormalTexture(icon)
	token.icon.link = GetLootRollItemLink(rollid)

	if canNeed	then token.needButt:Enable() 
				else token.needButt:Disable() end
	SetDesaturation(token.needButt:GetNormalTexture(), not canNeed)
	
	if canGreed	then token.greedButt:Enable() 
				else token.greedButt:Disable() end
	SetDesaturation(token.greedButt:GetNormalTexture(), not canGreed)
	
	if canDis	then token.disButt:Enable()
				else token.disButt:Disable() end
	SetDesaturation(token.disButt:GetNormalTexture(), not canDis)


	local bText, r, g, b = NekoLoot:GetItemBinding(token.icon.link)
	token.fsbind:SetVertexColor(r, g, b)
	token.fsbind:SetText(bText)
	token.fsloot:SetText(name)

	SetTokenColors(token, quality)

	token.status:SetMinMaxValues(0, time)
	token.status:SetValue(time)

	token:SetPoint("CENTER", WorldFrame, "CENTER")
	token:Show()
end


------
-- Handle CANCEL_LOOT_ROLL event
--
-- event			: event that should be handled
-- rollid			: rollid of the item
function NekoLootRoll:LootRollCanceled(event, rollid)
	for index, token in pairs(rollFrames) do
		if token.rollid == rollid then
			token.rollid = nil
			token.time = nil
			token:Hide()
			return
		end
	end
end


------
-- Find player, item and type of roll in a given (wow) chat msg.
-- Use def translation table on top
--
-- msg				: chat msg to parse
local function ParseRollChatMsg(msg)
	for _, rollType in pairs(rollTypes) do
		local _, _, player, item = string.find(msg, L[tostring(rollType)])
		if player and item and player ~= "Everyone" then
			-- swap item and player
			if ruRU and rollType ~= "pass" then
				return item, player, rollType
			-- change auto pass to pass
			elseif rollType == "passA" then
				return player, item, "pass"
			else
				return player, item, rollType
			end
		end
	end
end

------
-- Find roll infomation in given (wow) chat msg. If then update corresponding
-- loot token.
--
-- event			: event that should be handled
-- msg				: chat msg to parse
function NekoLootRoll:ChatMsgLoot(event, msg)
	local player, item, rollType = ParseRollChatMsg(msg)
	-- if anything is missed, escape
	if not player or not item or not rollType then return end

	for _, token in ipairs(rollFrames) do
		if token.rollid and token.icon.link == item and not token.rolls[player] then
			-- store rollType of player
			token.rolls[player] = rollType
			-- get roll count from frame and increment
			token[rollType]:SetText(tonumber(token[rollType]:GetText() + 1))
			return
		end
	end
end


------
-- Display a Tooltip for given frame.
--
-- frame			: parent frame of click event
function NekoLootRoll:ShowTooltip(frame)
	GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
	GameTooltip:SetText(frame.tipText)
	if frame:IsEnabled() == 0 then
		GameTooltip:AddLine(L["Cannot roll"])
	end
	
	for name, roll in pairs(frame.parent.rolls) do 
		if roll == rollTypes[frame.rollType] then 
			GameTooltip:AddLine(name, 1, 1, 1) 
		end
	end
	GameTooltip:Show()
end


------
-- Roll on an item.
--
-- frame			: parent frame of the click event
function NekoLootRoll:OnClickRoll(frame)
	if not frame.parent.rollid then return end
	RollOnLoot(frame.parent.rollid, frame.rollType)
end


------
--
--
function NekoLootRoll:OnClickToken()

end


------
-- EventHandler OnStatusUpdate of a roll frame.
-- Sets new time value and resizes highlight texture.
--
-- frame			: which frame must be updated
function NekoLootRoll:StatusUpdate(frame)
	-- when addon in config mode there is no rollid
	if not frame.parent.rollid then return end
	local time = GetLootRollTimeLeft(frame.parent.rollid)
	local perc = time / frame.parent.time
	frame.spark:SetPoint("CENTER", frame, "LEFT", perc * frame:GetWidth(), 0)
	frame:SetValue(time)
end


------
-- EventHandler OnEnter of a single roll item.
-- Shows the GameTooltip.
function NekoLootRoll:OnEnter(self)
	if self.link or self.icon.link then
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT", 32, 0)
		GameTooltip:SetHyperlink(self.link and self.link or self.icon.link)
		if IsShiftKeyDown() then
			GameTooltip_ShowCompareItem()
		end
		CursorUpdate(self)
	end
end


------
-- EventHandler OnLeave of a single roll item.
-- Hides the GameTooltip.
function NekoLootRoll:OnLeave(self)
	GameTooltip:Hide()
	ResetCursor()
end


------
-- EventHandler StartMoving of roll anchor.
-- Anchor will only be dragable, if unlocked.
function NekoLootRoll:StartMoving() 
	if not db.lock then
		rollAnc:StartMoving() 
	end
end


------
-- EventHandler StopMovingOrSizing of roll anchor.
function NekoLootRoll:StopMoving()
	if not db.lock then 
		rollAnc:StopMovingOrSizing()
		local point, _, relativePoint, xOff, yOff = rollAnc:GetPoint()

		self.db.profile.anchorPoint = point
		self.db.profile.parentAnchor = relativePoint
		self.db.profile.xOffSet = xOff
		self.db.profile.yOffSet = yOff
	end
	NekoLootRoll:ApplySettings()
end 
